<?php

namespace App\Http\Controllers\Application;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Events\NotificationCreatedEvent;
use Illuminate\Support\Facades\Auth;

use App\Http\Resources\CartsResource;
use App\Http\Resources\CartResource;
use App\Http\Resources\ProductsResource;

use App\Http\Resources\ARCartsResource;
use App\Http\Resources\ARCartResource;
use App\Http\Resources\ArProductsResource;

use App\Models\Category;
use App\Models\Product;
use App\Models\Cart;
use App\Models\CartOption;
use App\Models\User;
use App\Models\StaticContent;

use Carbon\Carbon;


class CartController extends Controller
{
    public function myCarts(Request $request){
        $userId = auth()->user()->id;
        $carts = Cart::where(['user_id'=>$userId,'ordered'=>0]);
        $products = Product::whereIn('branch_id',[auth()->user()->branch_id,3])->orderBy('created_at','DESC')->limit(5)->get();
        $subTotal = $carts->sum('price');
        $countItems =$carts->sum('amount');
        $carts = $carts->get();

        if($countItems == 0){
            return response()->json([
                'success' => false,
                'message' => $request->lang == 'en' ? 'Cart is Empty' : 'السلة فارغة'
            ], 400);
        }

        $static = StaticContent::first();

        $response = [
            'success' => true,
            'message' => "The Process is Completed Successfully",
            "data" => array(
                'carts' => $request->lang == 'en' ? CartsResource::make($carts) :  ARCartsResource::make($carts),
                "sub_total" => $subTotal,
                "delivery" => $static->delivery_fee,
                "total_price" => $subTotal +  $static->delivery_fee,
                "count_items" => $countItems,
                'products' =>  $request->lang == 'en' ? ProductsResource::make($products) :  ArProductsResource::make($products)
            )
        ];

        return response($response, 201); 
    }

    public function addToCart(Request $request,$productId){
        $arr =  [
            'options' => json_deCode($request->options)
        ];

        $validator = Validator::make($arr, [
            'options' => 'nullable|array',
            'options.*' => 'nullable|integer|exists:options,id',

        ]);

        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'message' => 'Please See Errors Parameter for all Errors.',
                'errors' => $validator->errors()
            ], 400);
        }

        $user = auth()->user();
        $product = Product::where('id',$productId)->whereIn('branch_id',[$user->branch_id,3])->first();

        if(!$product){
            return response()->json([
                'success' => false,
                'message' => $request->lang == 'en' ? 'Product Doesn\'t Exist' : 'المنتج غير موجود'
            ], 400);
        }

        $totalPrice = $product->discount_price == 0 ? $product->price : $product->discount_price;

        $cart = Cart::create([
            'amount' => 1,
            'product_id' => $productId,
            'user_id' => $user->id,
            'price' => $totalPrice,
        ]);

        if ($request->options) {
            $options =  json_decode($request->options);
            foreach($options as $extraOptionsId){
                $cartOption = CartOption::create([
                    'cart_id' => $cart->id,
                    'extra_option_id' => $extraOptionsId,
                ]);

                $totalPrice = $totalPrice + $cartOption->extraOption->price;
            }
            $totalPrice *= $cart->amount;
            $cart->update([
                'price' => $totalPrice,
            ]);
        }

        $carts = Cart::where(['user_id'=>$user->id,'ordered'=>0]);
        $subTotal = $carts->sum('price');
        $countItems =$carts->sum('amount');

        $response = [
            'success' => true,
            'message' => $request->lang == 'en' ? "Product Added to Cart" : "تمت اضافة المنتج الى السلة",
            'cart' => $request->lang == 'en' ? CartResource::make($cart) :  ARCartResource::make($cart),
            "sub_total" => $subTotal,
            "count_items" => $countItems,
        ];
        return response($response, 201);
    }

    public function changeAmount(Request $request,$productId){
        $arr =  [
            'options' => json_deCode($request->options)
        ];

        $validator = Validator::make($arr, [
            'options' => 'nullable|array',
            'options.*' => 'nullable|integer|exists:options,id',

        ]);

        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'message' => 'Please See Errors Parameter for all Errors.',
                'errors' => $validator->errors()
            ], 400);
        }

        $user = auth()->user();
        $carts = Cart::where('product_id',$productId)->where('user_id',$user->id)->where('ordered',0)->get();
        $isExist = 0;

        $product = Product::where('id',$productId)->whereIn('branch_id',[$user->branch_id,3])->first();

        if(!$product){
            return response()->json([
                'success' => false,
                'message' => $request->lang == 'en' ? 'Product Doesn\'t Exist' : 'المنتج غير موجود'
            ], 400);
        }

        $totalPrice = $product->discount_price == 0 ? $product->price : $product->discount_price;

        if ($request->options) {
            $options =  json_deCode($request->options);
            foreach($carts as $cart){
               $cartOption = $cart->options->pluck('extra_option_id')->toArray();
               if(array_diff($options,$cartOption) == array_diff($cartOption,$options)){
                $amount = $cart->amount + 1;
                $totalPrice = $cart->price;
                $totalPrice *= $amount;
                    $cart->update([
                        'amount' => $amount,
                        'price' => $totalPrice
                    ]);
                    $isExist = 1;
                    break;
                }
            }
        }else{
            foreach($carts as $cart){
                if($cart->options->count() == 0){
                    $amount = $cart->amount + 1;
                    $totalPrice = $cart->price;
                    $totalPrice *= $amount;
                    $cart->update([
                        'amount' => $amount,
                        'price' => $totalPrice
                    ]);
                    $isExist = 1;
                    break;
                }
            }
        }
        
        if($isExist ==0){
            $newCart = Cart::create([
                'amount' => 1,
                'product_id' => $productId,
                'user_id' => $user->id,
                'price' => $totalPrice,
            ]);
    
            if ($request->options) {
                $options =  json_deCode($request->options);
                foreach($options as $extraOptionsId){
                    $cartOption = CartOption::create([
                        'cart_id' => $newCart->id,
                        'extra_option_id' => $extraOptionsId,
                    ]);
    
                    $totalPrice = $totalPrice + $cartOption->extraOption->price;
                }
                $totalPrice *= $newCart->amount;
                $newCart->update([
                    'price' => $totalPrice,
                ]);
            }

            $cart = $newCart;
        }

        $carts = Cart::where(['user_id'=>$user->id,'ordered'=>0]);
        $subTotal = $carts->sum('price');
        $countItems =$carts->sum('amount');

        $response = [
            'success' => true,
            'message' => $request->lang == 'en' ? "Amount Updated" : "تم تحديث الكمية",
            'cart' => $request->lang == 'en' ? CartResource::make($cart) :  ARCartResource::make($cart),
            "total_price" => $subTotal,
            "count_items" => $countItems,
        ];
        return response($response, 201);
    }

    public function deleteCartItem(Request $request,$itemId){
        $user = auth()->user();
        $cart = Cart::where(['id'=>$itemId,'user_id'=>$user->id])->first();

        if(!$cart){
            return response()->json([
                'success' => false,
                'message' => $request->lang == 'en' ? 'Product Doesn\'t Found in Cart' :"المنتج غير موجود في السلة"
            ], 400);
        }

        if($cart->amount == 1) $cart->delete();

        $cartPrice = $cart->price/$cart->amount;
        $newPrice = $cart->price - $cartPrice;

        $cart->update([
            'price' => $newPrice,
            'amount' => $cart->amount - 1,
        ]);
        
        $carts = Cart::where(['user_id'=>$user->id,'ordered'=>0])->orderBy('id', 'desc');
        $subTotal = $carts->sum('price');
        $countItems =$carts->sum('amount');
        $lastCart = $carts->first();

        if($cart){
            if($lastCart){
                $response = [
                    'success' => true,
                    'message' => $request->lang == 'en' ? "Item Deleted Successfully" : "تم حذف المنتج بنجاح",
                    'cart' => $request->lang == 'en' ? CartResource::make($lastCart) :  ARCartResource::make($lastCart),
                    "total_price" => $subTotal,
                    "count_items" => $countItems,
                ];
                return response($response, 201);
            }else{
                $response = [
                    'success' => true,
                    'message' => $request->lang == 'en' ? "Item Deleted Successfully" : "تم حذف المنتج بنجاح",
                    'cart' => [],
                    "total_price" => $subTotal,
                    "count_items" => $countItems,
                ];
                return response($response, 201);
            }
        }else{
            $response = [
                'success' => false,
                'message' => $request->lang == 'en' ? "Item Doesn\'t Exist" : "المنتج غير موجود",
            ];
            return response($response, 400);
        }
    }

}
